(* somme des éléments de l *)
let rec somme (l: int list) : int = 
	match l with 
	| [] -> 0
	| x :: q -> x + somme q 

(* true si x est dans l, false sinon *)
let rec recherche (x: 'a) (l: 'a list) : bool =
	match l with 
	| [] -> true
	(* On ne peut pas mettre x dans le motif
	   car un motif sert à décrire une forme
	   et pas une valeur *)
	| y :: q -> if x = y then true else recherche x q


(* [x1; .. xn; y1; ...; ym],
   où l1 = [x1; ...; xn] et l2 = [y1; ...; ym] *)
 let rec concatener (l1: 'a list) (l2: 'a list) : 'a list =
 	match l1 with 
 	| [] -> l2
 	| x :: q -> x :: concatener q l2

(* concaténation de toutes les listes contenues dans l *)
 let rec multi_concat (ll: 'a list list) : 'a list =
 	match ll with 
 	| [] -> []
 	| l :: qq -> concatener l (multi_concat qq)

 (* [f x1; f x2; ...; f xn],
    où l = [x1; x2; ...; xn] *)
let rec map (f: 'a -> 'b) (l: 'a list) : 'b list =
	match l with 
	| [] -> []
	| x :: q -> f x :: map f q

(* concaténation de (l1 renversée) et l2 *)
let rec rev_concat (l1: 'a list) (l2: 'a list) : 'a list =
	match l1 with 
	| [] -> l2
	| x :: q -> rev_concat q (x::l2)

(* l dans l'ordre inverse *)
let reverse (l: 'a list) : 'a list =
	rev_concat l []

(* [0; ...; n-1] concaténé avec l *)
let rec range_concat (n: int) (l: int list) : int list =
	if n = 0 then l
	else range_concat (n-1) ((n-1)::l)

(* [0; ...; n-1] *)
let range (n: int) : int list = range_concat n []
